﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Loaded                       += Window_Loaded;
            this.tensionScrollBar.Scroll += tensionScrollBar_Scroll;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            DrawCurve(1.0);
        }

        #endregion
        #region 텐션 스크롤바 스크롤시 처리하기 - tensionScrollBar_Scroll(sender, e)

        /// <summary>
        /// 텐션 스크롤바 스크롤시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tensionScrollBar_Scroll(object sender, ScrollEventArgs e)
        {
            double tension = this.tensionScrollBar.Value / 10;

            this.tensionLabel.Content = tension.ToString("0.0");

            DrawCurve(tension);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 곡선 포인트 배열 구하기 - GetCurvePointArray(sourcePointArray, tension)

        /// <summary>
        /// 곡선 포인트 배열 구하기
        /// </summary>
        /// <param name="sourcePointArray">소스 포인트 배열</param>
        /// <param name="tension">텐션</param>
        /// <returns>곡선 포인트 배열</returns>
        private Point[] GetCurvePointArray(Point[] sourcePointArray, double tension)
        {
            if(sourcePointArray.Length < 2)
            {
                return null;
            }

            double controlScale = tension / 0.5 * 0.175;

            List<Point> targetPointList = new List<Point>();

            targetPointList.Add(sourcePointArray[0]);

            for(int i = 0; i < sourcePointArray.Length - 1; i++)
            {
                Point previousPoint = sourcePointArray[Math.Max(i - 1, 0)];
                Point sourcePoint   = sourcePointArray[i];
                Point nextPoint     = sourcePointArray[i + 1];
                Point nextNextPoint = sourcePointArray[Math.Min(i + 2, sourcePointArray.Length - 1)];

                double deltaX1 = nextPoint.X - previousPoint.X;
                double deltaY1 = nextPoint.Y - previousPoint.Y;

                Point point1 = sourcePointArray[i];
                Point point4 = nextPoint;

                double deltaX2 = nextPoint.X - previousPoint.X;
                double deltaY2 = nextPoint.Y - previousPoint.Y;

                Point point2 = new Point
                (
                    sourcePoint.X + controlScale * deltaX2,
                    sourcePoint.Y + controlScale * deltaY2
                );

                deltaX2 = nextNextPoint.X - sourcePoint.X;
                deltaY2 = nextNextPoint.Y - sourcePoint.Y;

                Point point3 = new Point
                (
                    nextPoint.X - controlScale * deltaX2,
                    nextPoint.Y - controlScale * deltaY2
                );

                targetPointList.Add(point2);
                targetPointList.Add(point3);
                targetPointList.Add(point4);
            }

            return targetPointList.ToArray();
        }

        #endregion
        #region 베지어 경로 구하기 - GetBezierPath(sourcePointArray)

        /// <summary>
        /// 베지어 경로 구하기
        /// </summary>
        /// <param name="sourcePointArray">소스 포인트 배열</param>
        /// <returns>베지어 경로</returns>
        private Path GetBezierPath(Point[] sourcePointArray)
        {
            Path path = new Path();

            PathGeometry pathGeometry = new PathGeometry();

            path.Data = pathGeometry;

            PathFigure pathFigure = new PathFigure();

            pathGeometry.Figures.Add(pathFigure);

            pathFigure.StartPoint = sourcePointArray[0];

            PathSegmentCollection pathSegmentCollection = new PathSegmentCollection();

            pathFigure.Segments = pathSegmentCollection;

            PointCollection pointCollection = new PointCollection(sourcePointArray.Length - 1);

            for(int i = 1; i < sourcePointArray.Length; i++)
            {
                pointCollection.Add(sourcePointArray[i]);
            }

            PolyBezierSegment polyBezierSegment = new PolyBezierSegment();

            polyBezierSegment.Points = pointCollection;

            pathSegmentCollection.Add(polyBezierSegment);

            return path;
        }

        #endregion
        #region 곡선 경로 구하기 - GetCurvePath(sourcePointArray, tension)

        /// <summary>
        /// 곡선 경로 구하기
        /// </summary>
        /// <param name="sourcePointArray">소스 포인트 배열</param>
        /// <param name="tension">텐션</param>
        /// <returns>곡선 경로</returns>
        private Path GetCurvePath(Point[] sourcePointArray, double tension)
        {
            if(sourcePointArray.Length < 2)
            {
                return null;
            }

            Point[] targetPointArray = GetCurvePointArray(sourcePointArray, tension);

            return GetBezierPath(targetPointArray.ToArray());
        }

        #endregion
        #region 곡선 그리기 - DrawCurve(tension)

        /// <summary>
        /// 곡선 그리기
        /// </summary>
        /// <param name="tension">텐션</param>
        private void DrawCurve(double tension)
        {
            this.canvas.Children.Clear();

            Point[] pointArray =
            {
                new Point(160,  50),
                new Point(400, 250),
                new Point(200, 270),
                new Point(400,  70)
            };

            Path path = GetCurvePath(pointArray, tension);

            path.Stroke          = Brushes.LightGreen;
            path.StrokeThickness = 5;

            this.canvas.Children.Add(path);

            foreach(Point point in pointArray)
            {
                Rectangle rectangle = new Rectangle();

                rectangle.Width           = 6;
                rectangle.Height          = 6;
                rectangle.Stroke          = Brushes.Black;
                rectangle.StrokeThickness = 1;
                rectangle.Fill            = Brushes.White;

                Canvas.SetLeft(rectangle, point.X - 3);
                Canvas.SetTop (rectangle, point.Y - 3);

                this.canvas.Children.Add(rectangle);
            }
        }

        #endregion
    }
}